# Generated by Django 2.2.2 on 2019-08-08 15:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_education', '0002_observationalert'),
    ]

    operations = [
        migrations.AddField(
            model_name='asyncprocess',
            name='process_type',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
