{% extends 'tom_common/base.html' %}
{% load bootstrap4 %}
{% block title %}Confirm delete{% endblock %}
{% block content %}
<form method="post" action="{% url 'tom_education:delete_dataproducts' %}">
    {% csrf_token %}
    <input type="hidden" name="next" value="{{ next }}" />
    <input type="hidden" name="product_pks" value="{{ product_pks }}" />
    <p>Are you sure you want to delete the following data products?</p>
    <ul>
        {% for prod in to_delete %}
            <li>{{ prod.data }}</li>
        {% endfor %}
    </ul>
    {% buttons %}
    <button type="submit" class="btn btn-danger">Confirm</button>
    {% endbuttons %}
</form>
{% endblock %}
