<input type="checkbox" name="{{ product.pk }}"
       class="dataproduct-checkbox
              {% for g in groups %} dpgroup-{{ g.pk }} {% endfor %}
              {% if reduced %}reduced{% endif %}" />
