{% extends 'tom_common/base.html' %}
{% load comments bootstrap4 tom_common_extras targets_extras observation_extras dataproduct_extras static cache tom_education_extras %}
{% block title %}{{ object.name }} Data View{% endblock %}
{% block additional_css %}
<link rel="stylesheet" href="{% static 'tom_targets/css/main.css' %}">
{% endblock %}
{% block content %}
<h2>&crarr;&nbsp;<a href="{% url 'tom_education:target_detail' object.pk %}">{{ object.name }}</a>: Data View
  {% if object.extra_fields.active %}
  <span class="badge badge-pill badge-info">Active</span>
  {% else %}
  <span class="badge badge-pill badge-secondary">Inactive</span>
  {% endif %}
  </h2>

  {% dataproduct_list_for_target object %}


{% endblock %}
