.thumbnail {
    text-align: center;
    padding: 1em;
    border-width: 1px;
    border-style: solid;
    border-color: white;  /* Must match page background color */
}

.thumbnail:hover {
    border-color: #007bff;  /* Boostrap blue */
}
