{% load bootstrap4 %}
{% load static tom_education_extras %}

<h2>Latest Images</h2>
<div class="row">
{% for p in products %}
<div class="col p-2">
  <img src="{{p.get_preview}}" />
  {% with p.extra_data|dataproduct_extrainfo as info %}
  {% if info %}
  <span class="datap_info info-{{info.siteid}}">
    {{info.siteid|upper}} {{info.telid}} {{info.exp_time|floatformat:"0" }}s in {{info.filter}}
  </span>
  {% endif %}
  {% endwith %}
</div>
{% endfor %}
</div>
